/******************************************************* 

		 eSAR Processor

   (Educational Synthetic Aperture Radar Processor)

		  developed by 
 
	    Department of Geophysics
	  Kangwon National Univeristy

 All right reserved by Kangwon National University.
 Contact: Dr. Hoonyol Lee (hoonyol@kangwon.ac.kr)

			* * *

The purpose of this educational SAR processor is to provide
some basic processing that is impractical for human calculation. 
This means that the user (mostly the student) should learn the meaning of
each SAR parameters and calculate them to be put into this eSAR processor.

This program requires the students:

   1) to provide the imput data in BIP,
   2) to provide all the information necessary from the header file,
   2) to calculate chrip parameters, 
   3) and to provide the Doppler paramters.

The students need to study the basic principles of SAR image formation,
and should be able to read, interpret, and derive the parameters
that is essential for SAR focusing and the following post-processing.     
The parameters need to be in the parameter file before running.
*************************************************************************/

#define VERSION "1.2" 
/************************************************************************
VERSION "1.2" <28 Nov 2004>
	1. ERMapper output header
	2. cfout in BIL
	3. maverage, multilook size from pam file 

VERSION "1.1" <31 May 2004>
	1. Range migration: Nearest Neighbour
	2. Control processing as well as output by 0, 1, 2 

VERSION "1.0" <21 May 2004>
	1. multilook: detect_image.c, maverage.c, multilook_fout.c
	2. ifft normalise with N.  

VERSION "0.9" (8 May 2004)
	1. all output in float, 
	2. raw data output added.

VERSION "0.8" (8 May 2004, INITIAL)
*************************************************************************/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>

#include "para.h"
#include "ma.c"
#include "load_para.c"
#include "load_data.c"
#include "range_chirp.c"
#include "fft.c"
#include "range_matched_filter.c"
#include "range_migration.c"
#include "azimuth_chirp.c"
#include "azimuth_matched_filter.c"
#include "gen_ers.c"
#include "cfout.c"
#include "detect_image.c"
#include "maverage.c"
#include "multilook_fout.c"

void usage();
void intro();

main(int argc, char *argv[])
{

	time_t clock_start, clock_end, clock_duration;

	intro();

	if(argc != 2){usage(); exit(0);} 

	clock_start=time(NULL);
 
	/* Load Parameters and Data */
	printf("Load Parameters and Data\n"); 
	load_para(argv[1]);
	if(oflag[1]>0) load_data();
	if(oflag[1]>1)
	   cfout(data, nazimuth_fft, nrange_fft, 1,1,0, "01_data_AP");


	/* Range Compression */
	if(oflag[2]>0)
	{
	   printf("\nRange Compression\n");
	   range_chirp();
	}	
	if(oflag[2]>1)
	   cfout(range_chirp_data, 1, nrange_fft, 0,1,0, "02_rchirp_IQ");	
	
	if(oflag[3]>0)
	{
	   printf(" forward FFT(range_chirp_data):\n");
	   fft(range_chirp_data, 1, nrange_fft, 0, 1);
	if(oflag[3]>1)
	   cfout(range_chirp_data, 1, nrange_fft, 0,1,0, "03_rchirp_fft_IQ");	
	}

	if(oflag[4]>0)
	{
	   printf(" forward FFT(data, range):\n");
	   fft(data, nazimuth_fft, nrange_fft, 0, 1);
	}
	if(oflag[4]>1)
	   cfout(data, nazimuth_fft, nrange_fft, 1,1,0, "04_data_rfft_AP");

	if(oflag[5]>0) range_matched_filter();	
	if(oflag[5]>1)
	   cfout(data, nazimuth_fft, nrange_fft, 1,1,0, "05_rmfilter_AP");

	if(oflag[6]>0)
	{	
	   printf(" inverse FFT(data, range):\n");
	   fft(data, nazimuth_fft, nrange_fft, 0, -1);
	}	
	if(oflag[6]>1)
	   cfout(data, nazimuth_fft, nrange_fft, 1,1,0, "06_rcomp_AP");


	/* Azimuth Compression */
	if(oflag[7]>0)
	{
	   printf("\nAzimuth Compression\n");
	   azimuth_chirp();
	}
	if(oflag[7]>1)
	   cfout(azimuth_chirp_data, 1, nazimuth_fft, 0,1,0, "07_azchirp_IQ");

	if(oflag[8]>0)
	{
	   printf(" forward FFT(azimuth_chirp_data):\n");
	   fft(azimuth_chirp_data, 1, nazimuth_fft, 0, 1);
	}	
	if(oflag[8]>1)
	   cfout(azimuth_chirp_data, 1, nazimuth_fft, 0,1,0, 
			"08_azchirp_fft_IQ");

	if(oflag[9]>0)
	{	
	   printf(" forward FFT(data, azimuth):\n");
	   fft(data, nazimuth_fft, nrange_fft, 1, 1);
	}	
	if(oflag[9]>1)
	   cfout(data, nazimuth_fft, nrange_fft, 1,1,0, "09_data_azfft_AP");

	if(oflag[10]>0) range_migration();
	if(oflag[10]>1)
	   cfout(data, nazimuth_fft, nrange_fft, 1,1,0, "10_rmigr_AP");

	if(oflag[11]>0) azimuth_matched_filter();
	if(oflag[11]>1)
	   cfout(data, nazimuth_fft, nrange_fft, 1,1,0, "11_azmfilter_AP");

	if(oflag[12]>0)
	{
	   printf(" inverse FFT(data, azimuth):\n");
	   fft(data, nazimuth_fft, nrange_fft, 1, -1);
	}	
	if(oflag[12]>1)
	   cfout(data, nazimuth_fft, nrange_fft, 0, 1, 0, "12_slc_IQ");
	if(oflag[13]>1)
	   cfout(data, nazimuth_fft, nrange_fft, 1, 1, 0, "13_slc_AP");

	/* Post-Processing */
	if(oflag[14]>0)
	{
	   printf("\nPost-Processing:\n");
	   detect_image(data, nazimuth_fft, nrange_fft);
	   maverage(data, nazimuth_fft, nrange_fft, 2, maverage_nx, maverage_ny);
	}	
	if(oflag[14]>1)
	   multilook_fout(data, nazimuth_fft, nrange_fft, 
			multilook_azsample, "14_MLC");

	clock_end=time(NULL);
	clock_duration= clock_end - clock_start;
	printf("\nProgram finished successfully (%d sec). \n", clock_duration);
}
 
void usage()
{
 	printf(" Usage: esar parameter_file\n");

}

void intro()
{
printf("\n");
printf(" *************************************************** \n"); 
printf("*	           eSAR Processor                   *\n");
printf("*  (educational Synthetic Aperture Radar Processor) *\n");
printf("* 		       v%s  			    *\n", VERSION);
printf("*     All right reserved by Dr. Hoonyol Lee,        *\n"); 
printf("* Dept. of Geophysics, Kangwon National University. *\n");
printf("* Email:hoonyol@kangwon.ac.kr, Tel:+82-33-250-8586  *\n");
printf(" *************************************************** \n"); 

}

   
